@@ -27,8 +27,9 @@ class Col : public Mat<eT>
   typedef eT                                elem_type;
   typedef typename get_pod_type<eT>::result pod_type;
   
-  static const bool is_col = true;
-  static const bool is_row = false;
+  static const bool is_col  = true;
+  static const bool is_row  = false;
+  static const bool is_xvec = false;
   
   inline          Col();
   inline          Col(const Col<eT>& X);
@@ -57,14 +58,15 @@ class Col : public Mat<eT>
   inline Col& operator=(Col&& m);
   #endif
   
-  inline explicit Col(const SpCol<eT>& X);
-  
   inline Col& operator=(const eT val);
   inline Col& operator=(const Col& m);
   
   template<typename T1> inline             Col(const Base<eT,T1>& X);
   template<typename T1> inline Col&  operator=(const Base<eT,T1>& X);
   
+  template<typename T1> inline explicit    Col(const SpBase<eT,T1>& X);
+  template<typename T1> inline Col&  operator=(const SpBase<eT,T1>& X);
+  
   inline Col(      eT* aux_mem, const uword aux_length, const bool copy_aux_mem = true, const bool strict = false);
   inline Col(const eT* aux_mem, const uword aux_length);
   
@@ -83,6 +85,8 @@ class Col : public Mat<eT>
   arma_inline const Op<Col<eT>,op_htrans> ht() const;
   arma_inline const Op<Col<eT>,op_strans> st() const;
   
+  arma_inline const Op<Col<eT>,op_strans> as_row() const;
+  
   arma_inline       subview_col<eT> row(const uword row_num);
   arma_inline const subview_col<eT> row(const uword row_num) const;
   
@@ -171,8 +175,6 @@ class Col<eT>::fixed : public Col<eT>
   
   arma_align_mem eT mem_local_extra[ (use_extra) ? fixed_n_elem : 1 ];
   
-  arma_inline void change_to_row();
-  
   
   public:
   
@@ -181,12 +183,13 @@ class Col<eT>::fixed : public Col<eT>
   typedef eT                                elem_type;
   typedef typename get_pod_type<eT>::result pod_type;
   
-  static const bool is_col = true;
-  static const bool is_row = false;
+  static const bool is_col  = true;
+  static const bool is_row  = false;
+  static const bool is_xvec = false;
   
-  static const uword n_rows = fixed_n_elem;
-  static const uword n_cols = 1;
-  static const uword n_elem = fixed_n_elem;
+  static const uword n_rows;  // value provided below the class definition
+  static const uword n_cols;  // value provided below the class definition
+  static const uword n_elem;  // value provided below the class definition
   
   arma_inline fixed();
   arma_inline fixed(const fixed<fixed_n_elem>& X);
@@ -250,4 +253,21 @@ class Col<eT>::fixed : public Col<eT>
 
 
 
+// these definitions are outside of the class due to bizarre C++ rules;
+// C++17 has inline variables to address this shortcoming
+
+template<typename eT>
+template<uword fixed_n_elem>
+const uword Col<eT>::fixed<fixed_n_elem>::n_rows = fixed_n_elem;
+
+template<typename eT>
+template<uword fixed_n_elem>
+const uword Col<eT>::fixed<fixed_n_elem>::n_cols = 1u;
+
+template<typename eT>
+template<uword fixed_n_elem>
+const uword Col<eT>::fixed<fixed_n_elem>::n_elem = fixed_n_elem;
+
+
+
 //! @}
