@@ -119,25 +119,18 @@ Col<eT>::Col(const SizeMat& s, const fill::fill_class<fill_type>& f)
 
 
 
-//! construct a column vector from specified text
 template<typename eT>
 inline
 Col<eT>::Col(const char* text)
+  : Mat<eT>(arma_vec_indicator(), 1)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 2;
-  
-  Mat<eT>::operator=(text);
-  
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
-  
-  access::rw(Mat<eT>::vec_state) = 1;
+  (*this).operator=(text);
   }
 
 
 
-//! construct a column vector from specified text
 template<typename eT>
 inline
 Col<eT>&
@@ -145,38 +138,32 @@ Col<eT>::operator=(const char* text)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 2;
+  Mat<eT> tmp(text);
   
-  Mat<eT>::operator=(text);
+  arma_debug_check( ((tmp.n_elem > 0) && (tmp.is_vec() == false)), "Mat::init(): requested size is not compatible with column vector layout" );
   
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
+  access::rw(tmp.n_rows) = tmp.n_elem;
+  access::rw(tmp.n_cols) = 1;
   
-  access::rw(Mat<eT>::vec_state) = 1;
+  (*this).steal_mem(tmp);
   
   return *this;
   }
 
 
 
-//! construct a column vector from specified text
 template<typename eT>
 inline
 Col<eT>::Col(const std::string& text)
+  : Mat<eT>(arma_vec_indicator(), 1)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 2;
-  
-  Mat<eT>::operator=(text);
-  
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
-  
-  access::rw(Mat<eT>::vec_state) = 1;
+  (*this).operator=(text);
   }
 
 
 
-//! construct a column vector from specified text
 template<typename eT>
 inline
 Col<eT>&
@@ -184,13 +171,14 @@ Col<eT>::operator=(const std::string& text)
   {
   arma_extra_debug_sigprint();
   
-  access::rw(Mat<eT>::vec_state) = 2;
+  Mat<eT> tmp(text);
   
-  Mat<eT>::operator=(text);
+  arma_debug_check( ((tmp.n_elem > 0) && (tmp.is_vec() == false)), "Mat::init(): requested size is not compatible with column vector layout" );
   
-  std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
+  access::rw(tmp.n_rows) = tmp.n_elem;
+  access::rw(tmp.n_cols) = 1;
   
-  access::rw(Mat<eT>::vec_state) = 1;
+  (*this).steal_mem(tmp);
   
   return *this;
   }
@@ -238,16 +226,11 @@ Col<eT>::operator=(const std::vector<eT>& x)
   template<typename eT>
   inline
   Col<eT>::Col(const std::initializer_list<eT>& list)
+    : Mat<eT>(arma_vec_indicator(), 1)
     {
     arma_extra_debug_sigprint();
     
-    access::rw(Mat<eT>::vec_state) = 2;
-    
-    Mat<eT>::operator=(list);
-    
-    std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
-    
-    access::rw(Mat<eT>::vec_state) = 1;
+    (*this).operator=(list);
     }
   
   
@@ -259,13 +242,14 @@ Col<eT>::operator=(const std::vector<eT>& x)
     {
     arma_extra_debug_sigprint();
     
-    access::rw(Mat<eT>::vec_state) = 2;
+    Mat<eT> tmp(list);
     
-    Mat<eT>::operator=(list);
+    arma_debug_check( ((tmp.n_elem > 0) && (tmp.is_vec() == false)), "Mat::init(): requested size is not compatible with column vector layout" );
     
-    std::swap( access::rw(Mat<eT>::n_rows), access::rw(Mat<eT>::n_cols) );
+    access::rw(tmp.n_rows) = tmp.n_elem;
+    access::rw(tmp.n_cols) = 1;
     
-    access::rw(Mat<eT>::vec_state) = 1;
+    (*this).steal_mem(tmp);
     
     return *this;
     }
@@ -336,23 +320,6 @@ Col<eT>::operator=(const std::vector<eT>& x)
 
 
 
-template<typename eT>
-inline
-Col<eT>::Col(const SpCol<eT>& X)
-  : Mat<eT>(arma_vec_indicator(), X.n_elem, 1, 1)
-  {
-  arma_extra_debug_sigprint_this(this);
-
-  arrayops::inplace_set(Mat<eT>::memptr(), eT(0), X.n_elem);
-
-  for(typename SpCol<eT>::const_iterator it = X.begin(); it != X.end(); ++it)
-    {
-    at(it.row()) = (*it);
-    }
-  }
-
-
-
 template<typename eT>
 inline
 Col<eT>&
@@ -409,6 +376,34 @@ Col<eT>::operator=(const Base<eT,T1>& X)
 
 
 
+template<typename eT>
+template<typename T1>
+inline
+Col<eT>::Col(const SpBase<eT,T1>& X)
+  : Mat<eT>(arma_vec_indicator(), 1)
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  Mat<eT>::operator=(X.get_ref());
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+Col<eT>&
+Col<eT>::operator=(const SpBase<eT,T1>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::operator=(X.get_ref());
+  
+  return *this;
+  }
+
+
+
 //! construct a column vector from a given auxiliary array of eTs
 template<typename eT>
 inline
@@ -549,6 +544,16 @@ Col<eT>::st() const
 
 
 
+template<typename eT>
+arma_inline
+const Op<Col<eT>,op_strans>
+Col<eT>::as_row() const
+  {
+  return Op<Col<eT>,op_strans>(*this);
+  }
+
+
+
 template<typename eT>
 arma_inline
 subview_col<eT>
@@ -959,7 +964,7 @@ Col<eT>::insert_rows(const uword row_num, const uword N, const bool set_to_zero)
       arrayops::copy( &(out_mem[row_num + N]), &(t_mem[row_num]), B_n_rows );
       }
     
-    if(set_to_zero == true)
+    if(set_to_zero)
       {
       arrayops::inplace_set( &(out_mem[row_num]), eT(0), N );
       }
@@ -1085,20 +1090,6 @@ Col<eT>::end_row(const uword row_num) const
 
 
 
-template<typename eT>
-template<uword fixed_n_elem>
-arma_inline
-void
-Col<eT>::fixed<fixed_n_elem>::change_to_row()
-  {
-  arma_extra_debug_sigprint();
-  
-  access::rw(Mat<eT>::n_cols) = fixed_n_elem;
-  access::rw(Mat<eT>::n_rows) = 1;
-  }
-
-
-
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
@@ -1128,7 +1119,7 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const fixed<fixed_n_elem>& X)
 
 template<typename eT>
 template<uword fixed_n_elem>
-inline
+arma_inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const subview_cube<eT>& X)
   : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
@@ -1160,7 +1151,7 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const fill::fill_class<fill_type>&)
 template<typename eT>
 template<uword fixed_n_elem>
 template<typename T1>
-inline
+arma_inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const Base<eT,T1>& A)
   : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
@@ -1174,7 +1165,7 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const Base<eT,T1>& A)
 template<typename eT>
 template<uword fixed_n_elem>
 template<typename T1, typename T2>
-inline
+arma_inline
 Col<eT>::fixed<fixed_n_elem>::fixed(const Base<pod_type,T1>& A, const Base<pod_type,T2>& B)
   : Col<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
@@ -1200,9 +1191,6 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const eT* aux_mem)
 
 
 
-//! NOTE: this function relies on
-//! Col::operator=(text), to change vec_state as well as swapping n_rows and n_cols,
-//! and Mat::init(), to check that the given vector will not have a different size than fixed_n_elem.
 template<typename eT>
 template<uword fixed_n_elem>
 inline
@@ -1211,16 +1199,11 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const char* text)
   {
   arma_extra_debug_sigprint_this(this);
   
-  change_to_row();
-  
   Col<eT>::operator=(text);
   }
 
 
 
-//! NOTE: this function relies on
-//! Col::operator=(text), to change vec_state as well as swapping n_rows and n_cols,
-//! and Mat::init(), to check that the given vector will not have a different size than fixed_n_elem.
 template<typename eT>
 template<uword fixed_n_elem>
 inline
@@ -1229,8 +1212,6 @@ Col<eT>::fixed<fixed_n_elem>::fixed(const std::string& text)
   {
   arma_extra_debug_sigprint_this(this);
   
-  change_to_row();
-  
   Col<eT>::operator=(text);
   }
 
@@ -1272,8 +1253,6 @@ Col<eT>::fixed<fixed_n_elem>::operator=(const char* text)
   {
   arma_extra_debug_sigprint();
   
-  change_to_row();
-  
   Col<eT>::operator=(text);
   
   return *this; 
@@ -1288,8 +1267,6 @@ Col<eT>::fixed<fixed_n_elem>::operator=(const std::string& text)
   {
   arma_extra_debug_sigprint();
   
-  change_to_row();
-  
   Col<eT>::operator=(text);
   
   return *this; 
