@@ -80,7 +80,24 @@ struct SpBase
   inline arma_warn_unused uword index_min() const;
   inline arma_warn_unused uword index_max() const;
   
+  inline arma_warn_unused bool is_symmetric() const;
+  inline arma_warn_unused bool is_symmetric(const typename get_pod_type<elem_type>::result tol) const;
+  
+  inline arma_warn_unused bool is_hermitian() const;
+  inline arma_warn_unused bool is_hermitian(const typename get_pod_type<elem_type>::result tol) const;
+  
   inline arma_warn_unused bool is_finite() const;
+  
+  inline const SpOp<derived,spop_vectorise_col> as_col() const;
+  inline const SpOp<derived,spop_vectorise_row> as_row() const;
+  
+  // TODO: is_empty()
+  // TODO: is_vec()
+  // TODO: is_rowvec()
+  // TODO: is_colvec()
+  // TODO: is_square()
+  // TODO: has_inf()
+  // TODO: has_nan()
   };
 
 
