@@ -30,8 +30,9 @@ class subview : public Base<eT, subview<eT> >
   
   arma_aligned const Mat<eT>& m;
   
-  static const bool is_row = false;
-  static const bool is_col = false;
+  static const bool is_row  = false;
+  static const bool is_col  = false;
+  static const bool is_xvec = false;
   
   const uword aux_row1;
   const uword aux_col1;
@@ -351,8 +352,9 @@ class subview_col : public subview<eT>
   typedef eT                                       elem_type;
   typedef typename get_pod_type<elem_type>::result pod_type;
   
-  static const bool is_row = false;
-  static const bool is_col = true;
+  static const bool is_row  = false;
+  static const bool is_col  = true;
+  static const bool is_xvec = false;
   
   const eT* colmem;
   
@@ -370,6 +372,8 @@ class subview_col : public subview<eT>
   arma_inline const Op<subview_col<eT>,op_htrans> ht() const;
   arma_inline const Op<subview_col<eT>,op_strans> st() const;
   
+  arma_inline const Op<subview_col<eT>,op_strans> as_row() const;
+  
   inline void fill(const eT val);
   inline void zeros();
   inline void ones();
@@ -441,8 +445,9 @@ class subview_row : public subview<eT>
   typedef eT                                       elem_type;
   typedef typename get_pod_type<elem_type>::result pod_type;
   
-  static const bool is_row = true;
-  static const bool is_col = false;
+  static const bool is_row  = true;
+  static const bool is_col  = false;
+  static const bool is_xvec = false;
   
   inline void operator= (const subview<eT>& x);
   inline void operator= (const subview_row& x);
@@ -458,6 +463,8 @@ class subview_row : public subview<eT>
   arma_inline const Op<subview_row<eT>,op_htrans> ht() const;
   arma_inline const Op<subview_row<eT>,op_strans> st() const;
   
+  arma_inline const Op<subview_row<eT>,op_strans> as_col() const;
+  
   inline eT  at_alt    (const uword i) const;
   
   inline eT& operator[](const uword i);
@@ -523,8 +530,9 @@ class subview_row_strans : public Base<eT, subview_row_strans<eT> >
   typedef eT                                       elem_type;
   typedef typename get_pod_type<elem_type>::result pod_type;
   
-  static const bool is_row = false;
-  static const bool is_col = true;
+  static const bool is_row  = false;
+  static const bool is_col  = true;
+  static const bool is_xvec = false;
   
   arma_aligned const subview_row<eT>& sv_row;
   
@@ -556,8 +564,9 @@ class subview_row_htrans : public Base<eT, subview_row_htrans<eT> >
   typedef eT                                       elem_type;
   typedef typename get_pod_type<elem_type>::result pod_type;
   
-  static const bool is_row = false;
-  static const bool is_col = true;
+  static const bool is_row  = false;
+  static const bool is_col  = true;
+  static const bool is_xvec = false;
   
   arma_aligned const subview_row<eT>& sv_row;
   
